@@ -52,6 +52,7 @@
 #include "rssh.h"
 #include "log.h"
 #include "rsshconf.h"
+#include "util.h"
 
 /*  MACRO DEFINITIONS */
 #define eat_assignment(x)	eat_char_token('=', x, FALSE, TRUE)
@@ -67,6 +68,9 @@ const char *keywords[] = {
 	"#",		/* start a comment */
 	"allowscp",
 	"allowsftp",
+	"allowcvs",
+	"allowrdist",
+	"allowrsync",
 	"chrootpath",
 	"logfacility",
 	"umask",
@@ -92,6 +96,15 @@ int process_allow_scp( ShellOptions_t *opts, const char *line,
 int process_allow_sftp( ShellOptions_t *opts, const char *line, 
 		        const int lineno );
 
+int process_allow_cvs( ShellOptions_t *opts, const char *line, 
+		        const int lineno );
+
+int process_allow_rdist( ShellOptions_t *opts, const char *line, 
+		        const int lineno );
+
+int process_allow_rsync( ShellOptions_t *opts, const char *line, 
+		        const int lineno );
+
 int get_token( const char *str, char *buf, const int buflen, 
 	       const bool colon, const bool ign_spc );
 
@@ -185,21 +198,36 @@ int process_config_line( ShellOptions_t	*opts,
 			return FALSE;
 		return TRUE;
 	case 3:
+		/* allow cvs */
+		if ( !(process_allow_cvs(opts, line + pos, lineno) ) )
+			return FALSE;
+		return TRUE;
+	case 4:
+		/* allow rdist */
+		if ( !(process_allow_rdist(opts, line + pos, lineno) ) )
+			return FALSE;
+		return TRUE;
+	case 5:
+		/* allow rsync */
+		if ( !(process_allow_rsync(opts, line + pos, lineno) ) )
+			return FALSE;
+		return TRUE;
+	case 6:
 		/* default chroot path */
 		if ( !(process_chroot_path(opts, line + pos, lineno) ) )
 			return FALSE;
 		return TRUE;
-	case 4:
+	case 7:
 		/* syslog log facility */
 		if ( !(process_log_facility(opts, line + pos, lineno) ) )
 			return FALSE;
 		return TRUE;
-	case 5:
+	case 8:
 		/* set the user's umask */
 		if ( !(process_umask(opts, line + pos, lineno) ) )
 			return FALSE;
 		return TRUE;
-	case 6:
+	case 9:
 		/* user */
 		if ( !(process_user(opts, line + pos, lineno) ) )
 			return FALSE;
@@ -427,6 +455,81 @@ int process_allow_sftp( ShellOptions_t *opts,
 }
 
 
+/* 
+ * process_allow_cvs() - make sure there are no tokens after the keyword,
+ *                        other than a possible comment.  If there are
+ *                        additional tokens other than comments, there is a
+ *                        syntax error, and FALSE is returned.  Otherwise, the
+ *                        line is ok, so opts are set to allow cvs, and TRUE
+ *                        is returned.
+ */
+int process_allow_cvs( ShellOptions_t *opts, 
+		       const char *line,
+		       const int lineno )
+{
+	int pos;
+
+	if ( !(pos = eat_comment(line)) ){
+		log_msg("line %d: syntax error parsing config file", lineno);
+		return FALSE;
+	}
+	log_set_priority(LOG_INFO);
+	log_msg("allowing cvs to all users");
+	opts->shell_flags |= RSSH_ALLOW_CVS;
+	return TRUE;
+}
+
+
+/* 
+ * process_allow_rdist() - make sure there are no tokens after the keyword,
+ *                        other than a possible comment.  If there are
+ *                        additional tokens other than comments, there is a
+ *                        syntax error, and FALSE is returned.  Otherwise, the
+ *                        line is ok, so opts are set to allow rdist, and TRUE
+ *                        is returned.
+ */
+int process_allow_rdist( ShellOptions_t *opts, 
+		       const char *line,
+		       const int lineno )
+{
+	int pos;
+
+	if ( !(pos = eat_comment(line)) ){
+		log_msg("line %d: syntax error parsing config file", lineno);
+		return FALSE;
+	}
+	log_set_priority(LOG_INFO);
+	log_msg("allowing rdist to all users");
+	opts->shell_flags |= RSSH_ALLOW_RDIST;
+	return TRUE;
+}
+
+
+/* 
+ * process_allow_rsync() - make sure there are no tokens after the keyword,
+ *                        other than a possible comment.  If there are
+ *                        additional tokens other than comments, there is a
+ *                        syntax error, and FALSE is returned.  Otherwise, the
+ *                        line is ok, so opts are set to allow rsync, and TRUE
+ *                        is returned.
+ */
+int process_allow_rsync( ShellOptions_t *opts, 
+		       const char *line,
+		       const int lineno )
+{
+	int pos;
+
+	if ( !(pos = eat_comment(line)) ){
+		log_msg("line %d: syntax error parsing config file", lineno);
+		return FALSE;
+	}
+	log_set_priority(LOG_INFO);
+	log_msg("allowing rsync to all users");
+	opts->shell_flags |= RSSH_ALLOW_RSYNC;
+	return TRUE;
+}
+
+
 int process_chroot_path( ShellOptions_t *opts, 
 		         const char *line,
 		         const int lineno )
@@ -698,6 +801,9 @@ int process_user( ShellOptions_t *opts,
 	int	optlen;			/* string length of user options */
 	bool	allow_scp;
 	bool	allow_sftp;
+	bool	allow_cvs;
+	bool	allow_rdist;
+	bool	allow_rsync;
 
 	/* make space for user options */
 	if ( !(temp = (char *)malloc(CFG_LINE_LEN + 1)) ){
@@ -758,7 +864,8 @@ int process_user( ShellOptions_t *opts,
 		log_msg("syntax error parsing user access, line %d", lineno);
 		return FALSE;
 	}
-	if ( !validate_access(axs, &allow_sftp, &allow_scp) ){
+	if ( !validate_access(axs, &allow_sftp, &allow_scp, &allow_cvs,
+			      &allow_rdist, &allow_rsync) ){
 		log_set_priority(LOG_ERR);
 		log_msg("syntax error parsing access bits, line %d", lineno);
 		return FALSE;
